@@ -27,6 +27,7 @@
  * \author Vaclav Petras (acyclic visitor pattern)
  */
 
+#include <algorithm>
 #include <fstream>
 #include <sstream>
 #include <iomanip>
@@ -607,9 +608,9 @@ void LocalNetwork::project_equations()
       }
   }
 
-  seznez_.erase(seznez_.begin(), seznez_.end());
-  seznez_.resize(pocet_neznamych_);
-  neznama_ nez;
+  unknowns_.erase(unknowns_.begin(), unknowns_.end());
+  unknowns_.resize(pocet_neznamych_);
+  Unknown unknown;
 
   for (ClusterList::iterator
          clptr=clusters.begin(); clptr!=clusters.end(); ++clptr)
@@ -623,10 +624,10 @@ void LocalNetwork::project_equations()
           const LocalPoint& station = PD[standpoint->station];
           if (station.active_xy())
             {
-              nez.cb  = standpoint->station;
-              nez.osn = standpoint;
-              nez.typ = 'R';
-              seznez_[standpoint->index_orientation()-1] = nez;
+              unknown.pid  = standpoint->station;
+              unknown.ori  = standpoint;
+              unknown.type = 'R';
+              unknowns_[standpoint->index_orientation()-1] = unknown;
           }
         }
 
@@ -636,17 +637,17 @@ void LocalNetwork::project_equations()
 
       if (b.active_xy() && b.index_y())
         {
-          nez.cb = (*i).first;
-          nez.osn =  0;
-          nez.typ = 'X';  seznez_[b.index_x()-1] = nez;
-          nez.typ = 'Y';  seznez_[b.index_y()-1] = nez;
+          unknown.pid = (*i).first;
+          unknown.ori =  0;
+          unknown.type = 'X';  unknowns_[b.index_x()-1] = unknown;
+          unknown.type = 'Y';  unknowns_[b.index_y()-1] = unknown;
         }
 
       if (b.active_z() && b.index_z())
         {
-          nez.cb = (*i).first;
-          nez.osn =  0;
-          nez.typ = 'Z';  seznez_[b.index_z()-1] = nez;
+          unknown.pid = (*i).first;
+          unknown.ori =  0;
+          unknown.type = 'Z';  unknowns_[b.index_z()-1] = unknown;
         }
     }
 
@@ -694,9 +695,10 @@ void LocalNetwork::project_equations()
         if (const int N = (*cluster)->activeObs())
         {
           int W = (*cluster)->covariance_matrix.bandWidth();
+          W = std::min(N-1, W);         // 2.16.4 ... added to fix the bug in W
           count++;
           msize += 1+(N+N-W)*(W+1)/2;   // += number of band matrix elements
-        }
+        }                               // 2.16.4 ... why plus one?
 
       GNU_gama::BlockDiagonal<> *bd = new GNU_gama::BlockDiagonal<>(count, msize);
 
@@ -1135,14 +1137,13 @@ void LocalNetwork::prepareProjectEquations()
 {
   int ind_0 = 0;
 
-  for (ClusterList::const_iterator
-         cluster=OD.clusters.begin(); cluster!=OD.clusters.end(); ++cluster)
-    if (const int N = (*cluster)->activeObs())
+  for (const auto cluster : OD.clusters)
+    if (const int N = cluster->activeObs())
         {
           Vec t(N);
-          CovMat C = (*cluster)->activeCov();
+          CovMat C = cluster->activeCov();
           C /= (m_0_apr_*m_0_apr_);        // covariances ==> cofactors
-          Adj::choldec(C);                     // cofactors   ==> weights
+          Adj::choldec(C);                 // cofactors   ==> weights
 
           for (int j=1; j<=A.cols(); j++)
             {
@@ -1327,12 +1328,14 @@ void LocalNetwork::vyrovnani_()
 }
 
 
-std::string LocalNetwork::export_xml()
+std::string LocalNetwork::export_xml(std::string version)
 {
-  if (!is_adjusted()) std::string();
-
   std::string xml =
-    "<?xml version=\"1.0\" ?>\n"
+    "<?xml version=\"1.0\" ?>\n";
+
+  if (!version.empty()) xml += version;  // optional xml comment
+
+  xml +=
     "<gama-local xmlns=\"http://www.gnu.org/software/gama/gama-local\">\n"
     "<network axes-xy=";
 
@@ -1360,9 +1363,9 @@ std::string LocalNetwork::export_xml()
     xml += "\n<description>" + description + "</description>\n";
 
   xml += "\n<parameters\n";
-  xml += "  sigma-apr=\"" + to_xmlstr(apriori_m_0()) + "\"\n";
-  xml += "  conf-pr=\""   + to_xmlstr(conf_pr())     + "\"\n";
-  xml += "  tol-abs=\""   + to_xmlstr(tol_abs())     + "\"\n";
+  xml += "  sigma-apr=\"" + to_xmlstr(apriori_m_0(), 8) + "\"\n";
+  xml += "  conf-pr=\""   + to_xmlstr(conf_pr(), 8)     + "\"\n";
+  xml += "  tol-abs=\""   + to_xmlstr(tol_abs(), 8)     + "\"\n";
   xml += "  sigma-act=\"";
   xml +=         m_0_apriori() ? "apriori\"\n" : "aposteriori\"\n";
   xml += "  angles=\"" + std::string(gons() ? "400" : "360") + "\"\n";
@@ -1447,9 +1450,8 @@ std::string LocalNetwork::export_xml()
             xml += " to=\"" + info.str_to + "\"";
             double fdh = obs->from_dh();
             double tdh = obs->to_dh();
-            if (info.xml_name != "s-distance") fdh = tdh = 0;
-            if (fdh) xml += " from_dh=\"" + to_xmlstr(fdh) + "\"";
-            if (tdh) xml +=   " to_dh=\"" + to_xmlstr(tdh) + "\"";
+            if (fdh) xml += " from_dh=\"" + to_xmlstr(fdh, 8) + "\"";
+            if (tdh) xml +=   " to_dh=\"" + to_xmlstr(tdh, 8) + "\"";
           }
           else if (!info.str_bs.empty()) {
             xml += " bs=\"" + info.str_bs + "\"";
@@ -1459,9 +1461,9 @@ std::string LocalNetwork::export_xml()
             double rdh = a->from_dh();
             double bdh = a->bs_dh();
             double fdh = a->fs_dh();
-            if (rdh) xml += " from_dh=\"" + to_xmlstr(rdh) + "\"";
-            if (bdh) xml +=   " bs_dh=\"" + to_xmlstr(bdh) + "\"";
-            if (fdh) xml +=   " fs_dh=\"" + to_xmlstr(fdh) + "\"";
+            if (rdh) xml += " from_dh=\"" + to_xmlstr(rdh, 8) + "\"";
+            if (bdh) xml +=   " bs_dh=\"" + to_xmlstr(bdh, 8) + "\"";
+            if (fdh) xml +=   " fs_dh=\"" + to_xmlstr(fdh, 8) + "\"";
           }
           xml += " val=\"" + info.str_val + "\"";
           xml += " stdev=\"" + info.str_stdev + "\"";
@@ -1554,8 +1556,8 @@ std::string LocalNetwork::export_xml()
 
           // double fdh = (*p)->from_dh(); ... unrealistic
           // double tdh = (*p)->to_dh();
-          // if (fdh) xml += " from_dh=\"" + xml_string(fdh) + "\"";
-          // if (tdh) xml +=   " to_dh=\"" + xml_string(tdh) + "\"";
+          // if (fdh) xml += " from_dh=\"" + xml_string(fdh, 8) + "\"";
+          // if (tdh) xml +=   " to_dh=\"" + xml_string(tdh, 8) + "\"";
           xml += " />\n";
         }
 
