@@ -25,6 +25,7 @@
 
 #include <cctype>
 #include <sstream>
+#include <limits>
 
 
 namespace GNU_gama { namespace local
@@ -123,7 +124,7 @@ void DisplayObservationVisitor::visit(Distance* obs)
   xml_name  = "distance";
   str_from  = obs->from().str();
   str_to    = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -133,9 +134,9 @@ void DisplayObservationVisitor::visit(Direction* obs)
   xml_name = "direction";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  double m = R2G*(obs->value());
+  double m = R2G*(obs->raw_value());
   if (lnet->gons())
-    str_val = to_xmlstr(m, 16);
+    str_val = to_xmlstr(m);
   else
     str_val = GNU_gama::gon2deg(m, 0, 4);
   str_stdev = to_xmlstr(obs->stdDev()*scale);
@@ -148,9 +149,9 @@ void DisplayObservationVisitor::visit(Angle* obs)
   str_from = obs->from().str();
   str_bs   = obs->bs().str();
   str_fs   = obs->fs().str();
-  double m = R2G*(obs->value());
+  double m = R2G*(obs->raw_value());
   if (lnet->gons())
-    str_val = to_xmlstr(m, 16);
+    str_val = to_xmlstr(m);
   else
     str_val = GNU_gama::gon2deg(m, 0, 4);
   str_stdev = to_xmlstr(obs->stdDev()*scale);
@@ -162,7 +163,7 @@ void DisplayObservationVisitor::visit(H_Diff* obs)
   xml_name = "dh";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -172,7 +173,7 @@ void DisplayObservationVisitor::visit(S_Distance* obs)
   xml_name = "s-distance";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -182,7 +183,7 @@ void DisplayObservationVisitor::visit(Z_Angle* obs)
   xml_name = "z-angle";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  double m = R2G*(obs->value());
+  double m = R2G*(obs->raw_value());
   if (lnet->gons())
     str_val =  to_xmlstr(m);
   else
@@ -195,7 +196,7 @@ void DisplayObservationVisitor::visit(X* obs)
   clear();
   xml_name = "x";
   str_from = obs->from().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -204,7 +205,7 @@ void DisplayObservationVisitor::visit(Y* obs)
   clear();
   xml_name = "y";
   str_from = obs->from().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -213,7 +214,7 @@ void DisplayObservationVisitor::visit(Z* obs)
   clear();
   xml_name = "z";
   str_from = obs->from().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -223,7 +224,7 @@ void DisplayObservationVisitor::visit(Xdiff* obs)
   xml_name = "dx";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -233,7 +234,7 @@ void DisplayObservationVisitor::visit(Ydiff* obs)
   xml_name = "dy";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -243,7 +244,7 @@ void DisplayObservationVisitor::visit(Zdiff* obs)
   xml_name = "dz";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  str_val   = to_xmlstr(obs->value(), 16);
+  str_val   = to_xmlstr(obs->raw_value());
   str_stdev = to_xmlstr(obs->stdDev());
 }
 
@@ -253,9 +254,9 @@ void DisplayObservationVisitor::visit(Azimuth* obs)
   xml_name = "azimuth";
   str_from = obs->from().str();
   str_to   = obs->to().str();
-  double m = R2G*(obs->value());
+  double m = R2G*(obs->raw_value());
   if (lnet->gons())
-    str_val = to_xmlstr(m, 16);
+    str_val = to_xmlstr(m);
   else
     str_val = GNU_gama::gon2deg(m, 0, 4);
   str_stdev = to_xmlstr(obs->stdDev()*scale);
